package wusthelper.web.service.campus;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.github.yitter.idgen.YitIdHelper;
import org.springframework.stereotype.Service;
import wusthelper.code.ServiceCode;
import wusthelper.data.dao.mapper.StudentMapper;
import wusthelper.data.entity.Student;
import wusthelper.web.data.entity.CookieType;
import wusthelper.web.exception.ServiceException;
import wusthelper.web.rpc.graduate.GraduateStudentInfoRemote;
import wusthelper.web.rpc.undergrad.UndergradStudentInfoRemote;
import wusthelper.web.service.campus.graduate.GraduateCookieService;
import wusthelper.web.service.campus.undergrad.UndergradCookieService;
import wusthelper.web.service.cookie.CookieManager;
import wusthelper.web.util.Converter;
import wusthelper.web.util.PasswordCodec;

/**
 * 通用的用户信息服务，包括登录和（已存储的）用户信息获取等，同时自动存入新用户信息
 */
@Service
public class GeneralUserService {
    private final StudentMapper studentMapper;
    private final PasswordCodec passwordCodec;

    private final UndergradCookieService undergradCookieService;
    private final GraduateCookieService graduateCookieService;

    private final GraduateStudentInfoRemote graduateStudentInfoRemote;
    private final UndergradStudentInfoRemote undergradStudentInfoRemote;


    public GeneralUserService(StudentMapper studentMapper,
                              PasswordCodec passwordCodec,

                              UndergradCookieService undergradCookieService,
                              GraduateCookieService graduateCookieService,

                              GraduateStudentInfoRemote graduateStudentInfoRemote,
                              UndergradStudentInfoRemote undergradStudentInfoRemote) {

        this.studentMapper = studentMapper;
        this.passwordCodec = passwordCodec;

        this.undergradCookieService = undergradCookieService;
        this.graduateCookieService = graduateCookieService;

        this.graduateStudentInfoRemote = graduateStudentInfoRemote;
        this.undergradStudentInfoRemote = undergradStudentInfoRemote;
    }

    public enum UserType {Undergrad, Graduate}

    /**
     * 登录验证并获取用户信息，同时自动存库
     *
     * @param username 用户名（即学号）
     * @param password 密码（明文原文）
     * @param userType 用户类型，现在有本科生和研究生
     * @return 用户信息
     */
    public Student login(String username, String password, UserType userType) {
        var cookie = switch (userType) {
            case Undergrad -> undergradCookieService.getLoginCookie(username, password);
            case Graduate -> graduateCookieService.getLoginCookie(username, password);
        };

        var cookieType = switch (userType) {
            case Undergrad -> CookieType.Undergrad;
            case Graduate -> CookieType.Graduate;
        };

        // 保存cookie，并获取用户信息并存库（基本的操作，非异步，信息获取失败意味登陆失败）
        CookieManager.store(username, cookie, cookieType);
        return this.saveOrUpdateUser(username, cookie, password, userType);
    }

    private Student saveOrUpdateUser(String username, String cookie, String password, UserType userType) {
        var rpcResp = switch (userType) {
            case Undergrad -> undergradStudentInfoRemote.get(cookie);
            case Graduate -> graduateStudentInfoRemote.get(cookie);
        };

        if (rpcResp.code() != ServiceCode.Ok) {
            ServiceException.of(rpcResp.code());
        }

        var studentInfo = rpcResp.data();
        var student = Converter.convert(studentInfo);
        if (student.getStuNum() == null) {
            student.setStuNum(username);
        }

        var query = new QueryWrapper<Student>()
                .eq("stu_num", username)
                .eq("deleted", 0);

        // （按学号）检查用户信息是否已经存在数据库中，已存在则更新，否则新增信息
        var studentExists = studentMapper.exists(query);
        if (studentExists) {
            studentMapper.update(student, query);
        } else {
            student.setId(YitIdHelper.nextId());
            var encrypted = passwordCodec.encode(password);
            student.setJwcPwd(encrypted);
            if (userType == UserType.Undergrad) {
                student.setLibPwd(encrypted);
            }

            studentMapper.insert(student);
        }

        return student;
    }
}
