package wusthelper.web.service.cookie;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import wusthelper.code.ServiceCode;
import wusthelper.data.dao.mapper.StudentMapper;
import wusthelper.data.entity.Student;
import wusthelper.web.data.entity.CookieType;
import wusthelper.web.exception.ServiceException;
import wusthelper.web.service.campus.graduate.GraduateCookieService;
import wusthelper.web.service.campus.undergrad.UndergradCookieService;
import com.github.benmanes.caffeine.cache.Cache;
import com.github.benmanes.caffeine.cache.Caffeine;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Service;

import java.util.concurrent.TimeUnit;

@Service
@Scope("singleton")
public class CookieManager {
    private static final Cache<String, String> cookiePool = Caffeine.newBuilder()
            .expireAfterWrite(2, TimeUnit.HOURS)
            .initialCapacity(256)
            .maximumSize(2048)
            .build();

    private final UndergradCookieService undergradCookieService;
    private final GraduateCookieService graduateCookieService;

    private final StudentMapper studentMapper;

    public CookieManager(UndergradCookieService undergradCookieService,
                         GraduateCookieService graduateCookieService,
                         StudentMapper studentMapper) {

        this.undergradCookieService = undergradCookieService;
        this.graduateCookieService = graduateCookieService;

        this.studentMapper = studentMapper;
    }

    public String getCookie(String username, CookieType cookieType) {
        return getCookie(username, cookieType, false);
    }

    public String getCookie(String username, CookieType cookieType, boolean forceRefresh) {
        String cacheKey = String.format("%s:%d", username, cookieType.getCode());
        String cookie = cookiePool.getIfPresent(cacheKey);

        boolean valid = cookie != null && checkCookie(cookie, cookieType) && !forceRefresh;
        if (valid) {
            return cookie;
        }

        cookie = this.refreshCookie(username, cookieType);
        cookiePool.put(cacheKey, cookie);

        return cookie;
    }

    public static void store(String username, String cookie, CookieType cookieType) {
        String cacheKey = String.format("%s:%d", username, cookieType.getCode());
        cookiePool.put(cacheKey, cookie);
    }

    public boolean checkCookie(String cookie, CookieType cookieType) {
        return switch (cookieType) {
            case Undergrad -> undergradCookieService.checkCookie(cookie);
            case Graduate -> graduateCookieService.checkCookie(cookie);
            default -> false;
        };
    }

    public String refreshCookie(String username, CookieType cookieType) {
        var student = studentMapper.selectOne(new QueryWrapper<Student>()
                .eq("stuNum", username)
                .eq("deleted", 0)
        );

        if (student == null) {
            ServiceException.of(ServiceCode.UserNotExists);
        }

        String password = switch (cookieType) {
            case Undergrad, Graduate -> student.getJwcPwd();
            case Library -> student.getLibPwd();
            case Physics -> student.getWlsyPwd();
        };

        if (password == null) {
            ServiceException.of(ServiceCode.ParamWrong);
        }

        return switch (cookieType) {
            case Undergrad -> undergradCookieService.getLoginCookie(username, password);
            case Graduate -> graduateCookieService.getLoginCookie(username, password);
            default -> null;
        };
    }
}