package wusthelper.internal.rpc;

import cn.hutool.core.thread.ThreadUtil;
import cn.hutool.core.util.RandomUtil;
import cn.wustlinghang.mywust.network.Requester;
import wusthelper.internal.rpc.client.ConsulClient;
import wusthelper.internal.rpc.client.FrpcClient;
import wusthelper.internal.rpc.config.FrpConfig;
import wusthelper.internal.rpc.config.RegisterConfig;
import wusthelper.internal.rpc.entity.RegisterRequestBody;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import lombok.extern.slf4j.Slf4j;

@Slf4j
public class FrpConsulRegister {

    private final String serviceId;

    private final RegisterConfig registerConfig;

    private final FrpcClient frpcClientClient;

    private final ConsulClient consulClient;

    private boolean registered = false;

    public FrpConsulRegister(RegisterConfig registerConfig, FrpConfig frpConfig,
                             Requester requester, ObjectMapper objectMapper) {

        this.serviceId = RandomUtil.randomString(8);
        this.registerConfig = registerConfig;

        this.frpcClientClient = new FrpcClient(frpConfig, requester, objectMapper);
        this.consulClient = new ConsulClient(this.serviceId, registerConfig, objectMapper, requester);
    }

    public void register() {
        ThreadUtil.execute(() -> {
            try {
                log.info("连接frp并注册consul...");
                this.doRegister();
                this.registered = true;
                log.info("注册完毕");
            } catch (Exception e) {
                log.warn("注册服务时发生异常：", e);
                log.warn("不进行转发注册，直接服务");
            }
        });

        Runtime.getRuntime().addShutdownHook(new Thread(this::onShutdown, "ShutdownHookThread"));
    }

    private void onShutdown() {
        try {
            if (this.registered) {
                log.info("注销服务...");
                this.consulClient.removeService();
                this.frpcClientClient.removeProxy();
                this.registered = false;
                log.info("服务注销完毕");
            }
        } catch (Exception e) {
            log.warn("注销服务时发生异常：", e);
        }
    }

    private void doRegister() throws Exception {
        frpcClientClient.addFrpProxy(
                registerConfig.getServiceName(), serviceId,
                "127.0.0.1", registerConfig.getLocalServicePort()
        );

        String remoteAddress;
        String[] remote;
        int retry = 0;
        do {
            // 先睡个0.5秒等待连接成功再获取状态读端口
            ThreadUtil.sleep(500);

            JsonNode proxy = frpcClientClient.getProxyStatus(registerConfig.getServiceName(), serviceId);
            remoteAddress = proxy.path("remote_addr")
                    .asText("127.0.0.1:" + registerConfig.getLocalServicePort());
            remote = remoteAddress.split(":");
            retry++;
        } while (retry < 3 && remote.length < 2);
        if (retry == 3) {
            throw new Exception("获取frp隧道信息重试超时，请手动添加隧道和注册中心");
        }

        RegisterRequestBody.HealthCheckOption healthCheckOption = RegisterRequestBody.HealthCheckOption.builder()
                .http("http://" + remoteAddress + "/health")
                .interval("3s")
                .timeout("5s")
                .build();

        this.consulClient.addService(remote[0], Integer.parseInt(remote[1]), healthCheckOption);
    }
}